@@ -25,16 +25,28 @@ __all__ = [
     "Discard",
     "add_validator",
     "validate",
-    "validator"
+    "validator",
 ]
 
 import apischema.std_types  # Populate custom handlers for stdlib
+
 from .alias import alias
-from .conversion import (converter, field_input_converter, field_output_converter,
-                         inout_model, input_converter, output_converter)
+from .conversion import (
+    converter,
+    field_input_converter,
+    field_output_converter,
+    inout_model,
+    input_converter,
+    output_converter,
+)
 from .data import from_data, items_to_data, to_data
-from .fields import (get_fields, get_fields_set, mark_set_fields, unmark_set_fields,
-                     with_fields_set)
+from .fields import (
+    get_fields,
+    get_fields_set,
+    mark_set_fields,
+    unmark_set_fields,
+    with_fields_set,
+)
 from .ignore import Ignored
 from .json_schema import build_input_schema, build_output_schema
 from .json_schema.types import JSONSchema
