@@ -1,16 +1,28 @@
+import sys
 from collections import defaultdict
-from dataclasses import (MISSING, dataclass, field, fields, is_dataclass)
-from inspect import (Parameter, isclass, isgeneratorfunction,
-                     signature)
+from inspect import Parameter, isclass, isgeneratorfunction, signature
 from logging import getLogger
-from types import MappingProxyType, MethodType
-from typing import (Any, Callable, Dict, Generator, Generic, Mapping,
-                    Optional, Tuple, Type, TypeVar, Union, cast,
-                    get_type_hints)
+from types import MethodType
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Generic,
+    Mapping,
+    Optional,
+    Tuple,
+    Type,
+    TypeVar,
+    Union,
+    cast,
+)
+
+from dataclasses import MISSING, dataclass, field
 
 from apischema.properties import properties
-from apischema.types import (DictWithUnion, Metadata, TYPED_ORIGINS)
-from apischema.typing import _GenericAlias
+from apischema.types import DictWithUnion, Metadata
+from apischema.typing import _GenericAlias, get_type_hints
 from apischema.utils import GeneratorValue, PREFIX, as_dict, to_camel_case
 from apischema.validation.errors import build_from_errors
 from apischema.visitor import NOT_CUSTOM, NotCustom
@@ -27,10 +39,9 @@ Param = TypeVar("Param")
 Return = TypeVar("Return")
 
 
-def check_converter(converter: Converter,
-                    param: Optional[Type[Param]],
-                    ret: Optional[Type[Return]]
-                    ) -> Tuple[Type[Param], Type[Return]]:
+def check_converter(
+    converter: Converter, param: Optional[Type[Param]], ret: Optional[Type[Return]]
+) -> Tuple[Type[Param], Type[Return]]:
     try:
         parameters = iter(signature(converter).parameters.values())
     except ValueError:  # builtin types
@@ -45,10 +56,13 @@ def check_converter(converter: Converter,
             raise TypeError("converter must have at least one parameter")
         for p in parameters:
             if p.default is Parameter.empty and p.kind not in (
-                    Parameter.VAR_POSITIONAL, Parameter.VAR_KEYWORD):
-                raise TypeError("converter must have at most one parameter "
-                                "without default")
-        types = get_type_hints(converter)
+                Parameter.VAR_POSITIONAL,
+                Parameter.VAR_KEYWORD,
+            ):
+                raise TypeError(
+                    "converter must have at most one parameter " "without default"
+                )
+        types = get_type_hints(converter, include_extras=True)
         if param is None:
             try:
                 param = types.pop(first.name)
@@ -62,8 +76,9 @@ def check_converter(converter: Converter,
     return cast(Type[Param], param), cast(Type[Return], ret)
 
 
-def handle_potential_validation(ret: Type, converter: Converter
-                                ) -> Tuple[Type, Converter]:
+def handle_potential_validation(
+    ret: Type, converter: Converter
+) -> Tuple[Type, Converter]:
     if not isgeneratorfunction(converter):
         return ret, converter
 
@@ -74,7 +89,7 @@ def handle_potential_validation(ret: Type, converter: Converter
             raise build_from_errors(errors)
         return generator.value
 
-    if isinstance(ret, _GenericAlias) and ret.__origin__ == Generator:
+    if getattr(ret, "__origin__", None) == Generator:
         return ret.__args__[2], wrapper
     else:
         return ret, wrapper
@@ -111,22 +126,58 @@ def handle_method(decorator: Func) -> Func:
 Cls = TypeVar("Cls", bound=Type)
 Other = TypeVar("Other", bound=Type)
 
+if sys.version_info >= (3, 7):
+    import collections.abc
+    import re
+    from typing import (
+        List,
+        AbstractSet,
+        Set,
+        Dict,
+        Iterable,
+        Collection,
+        Sequence,
+        MutableSequence,
+        Pattern,
+    )
 
-def _get_origin(cls: _GenericAlias) -> Type:  # type: ignore
-    return TYPED_ORIGINS.get(cls.__origin__, cls.__origin__)  # type: ignore
+    TYPED_ORIGINS = {
+        tuple: Tuple,
+        list: List,
+        frozenset: AbstractSet,
+        set: Set,
+        dict: Dict,
+        collections.abc.Iterable: Iterable,
+        collections.abc.Collection: Collection,
+        collections.abc.Sequence: Sequence,
+        collections.abc.MutableSequence: MutableSequence,
+        collections.abc.Set: AbstractSet,
+        collections.abc.MutableSet: Set,
+        re.Pattern: Pattern,
+    }
+
+    def _get_origin(cls: _GenericAlias) -> Type:  # type: ignore
+        return TYPED_ORIGINS.get(cls.__origin__, cls.__origin__)  # type: ignore
+
+
+else:
+
+    def _get_origin(cls: _GenericAlias) -> Type:  # type: ignore
+        return cls.__origin__  # type: ignore
 
 
 # If A[str, T] and A[T, int], what to choose for A[str, int]? None of them.
 
+
 def substitute_generic_args(cls: Cls, substitution: Mapping) -> Cls:
-    if not isinstance(cls, _GenericAlias):
+    if getattr(cls, "__origin__", None) is None:
         return cls
     args = tuple(substitution.get(arg, arg) for arg in cls.__args__)
     return _get_origin(cls)[args]
 
 
 def substitute_type_vars(base: Cls, other: Other) -> Tuple[Cls, Other]:
-    if not isinstance(base, _GenericAlias):
+    if getattr(base, "__origin__", None) is None:
         return base, other
     substitution = {
         arg: param
@@ -140,21 +191,24 @@ def substitute_type_vars(base: Cls, other: Other) -> Tuple[Cls, Other]:
     if all(isinstance(arg, TypeVar) for arg in base.__args__):  # type: ignore
         return cast(Tuple[Cls, Other], (_get_origin(base), new_other))
     else:
-        return cast(Tuple[Cls, Other],
-                    (substitute_generic_args(base, substitution), new_other))
+        return cast(
+            Tuple[Cls, Other], (substitute_generic_args(base, substitution), new_other)
+        )
 
 
 @handle_method
 def converter(function: Func, param: Type = None, ret: Type = None) -> Func:
     param, ret = check_converter(function, param, ret)
     param, ret = substitute_type_vars(param, ret)
+    if hasattr(param, "__supertype__"):
+        # For performance reason, serialization is based on objects class
+        raise TypeError("NewType cannot be used with converter/output_converter")
     _converters[param][ret] = function
     return function
 
 
 @handle_method
-def input_converter(function: Func, param: Type = None, ret: Type = None
-                    ) -> Func:
+def input_converter(function: Func, param: Type = None, ret: Type = None) -> Func:
     param, ret = check_converter(function, param, ret)
     ret, function_ = handle_potential_validation(ret, function)
     ret, param = substitute_type_vars(ret, param)
@@ -163,8 +217,7 @@ def input_converter(function: Func, param: Type = None, ret: Type = None
 
 
 @handle_method
-def output_converter(function: Func, param: Type = None, ret: Type = None
-                     ) -> Func:
+def output_converter(function: Func, param: Type = None, ret: Type = None) -> Func:
     param, ret = check_converter(function, param, ret)
     param, ret = substitute_type_vars(param, ret)
     _output_converters[param] = ret, function
@@ -172,8 +225,7 @@ def output_converter(function: Func, param: Type = None, ret: Type = None
     return function
 
 
-def inout_model(model: Type, *, replace_new_type: bool = True
-                ) -> Callable[[Cls], Cls]:
+def inout_model(model: Type, *, replace_new_type: bool = True) -> Callable[[Cls], Cls]:
     def decorator(cls: Cls) -> Cls:
         input_converter(cls, model, cls)
         out_conv = model
@@ -189,8 +241,7 @@ Arg = TypeVar("Arg")
 
 
 class InputVisitorMixin(Generic[Arg, Return]):
-    def _custom(self, cls: Type, custom: Dict[Type, Converter],
-                arg: Arg) -> Return:
+    def _custom(self, cls: Type, custom: Dict[Type, Converter], arg: Arg) -> Return:
         raise NotImplementedError()
 
     def custom(self, cls: Type, arg: Arg) -> Union[Return, NotCustom]:
@@ -209,8 +260,7 @@ class OutputVisitorMixin(Generic[Arg, Return]):
             else:
                 self.converters[cls] = out, _converters[cls][out]
 
-    def _custom(self, cls: Type, custom: Tuple[Type, Converter],
-                arg: Arg) -> Return:
+    def _custom(self, cls: Type, custom: Tuple[Type, Converter], arg: Arg) -> Return:
         raise NotImplementedError()
 
     def custom(self, cls: Type, arg: Arg) -> Union[Return, NotCustom]:
@@ -225,8 +275,9 @@ INPUT_METADATA = f"{PREFIX}input"
 OUTPUT_METADATA = f"{PREFIX}output"
 
 
-def field_input_converter(converter: Callable[[Any], Any], param: Type = None,
-                          ret: Type = None) -> Metadata:
+def field_input_converter(
+    converter: Callable[[Any], Any], param: Type = None, ret: Type = None
+) -> Metadata:
     try:
         param, ret = check_converter(converter, param, ret)
     except NameError:
@@ -236,8 +287,9 @@ def field_input_converter(converter: Callable[[Any], Any], param: Type = None,
     return DictWithUnion({INPUT_METADATA: (param, converter)})
 
 
-def field_output_converter(converter: Callable[[Any], Any], param: Type = None,
-                           ret: Type = None) -> Metadata:
+def field_output_converter(
+    converter: Callable[[Any], Any], param: Type = None, ret: Type = None
+) -> Metadata:
     try:
         param, ret = check_converter(converter, param, ret)
     except NameError:
@@ -245,39 +297,19 @@ def field_output_converter(converter: Callable[[Any], Any], param: Type = None,
     return DictWithUnion({OUTPUT_METADATA: (ret, converter)})
 
 
-# Because of NameError when type is recursive
-def resolve_fieds_converters(cls: Type):
-    assert is_dataclass(cls)
-    for field in fields(cls):
-        metadata = dict(field.metadata)
-        if INPUT_METADATA in metadata:
-            param, converter = metadata[INPUT_METADATA]
-            param, _ = check_converter(converter, param, ...)  # type: ignore
-            param, _ = substitute_type_vars(param, ...)  # type: ignore
-            _, converter = handle_potential_validation(..., converter)  # type: ignore
-            metadata[INPUT_METADATA] = param, converter
-        if OUTPUT_METADATA in metadata:
-            ret, converter = metadata[OUTPUT_METADATA]
-            _, ret = check_converter(converter, ..., ret)  # type: ignore
-            ret, _ = substitute_type_vars(ret, ...)  # type: ignore
-            metadata[OUTPUT_METADATA] = ret, converter
-        field.metadata = MappingProxyType(metadata)
-
-
 def converter_from_raw(func: Callable) -> Converter:
-    types = get_type_hints(func)
+    types = get_type_hints(func, include_extras=True)
     sig = signature(func)
     annotations: Dict[str, Any] = {}
     fields: Dict[str, Any] = {}
     kwargs = None
     for name, param in sig.parameters.items():
         if param.kind == Parameter.POSITIONAL_ONLY:
-            raise TypeError(f"Forbidden positional-only parameter")
+            raise TypeError("Forbidden positional-only parameter")
         if param.kind == Parameter.VAR_POSITIONAL:
-            raise TypeError(f"Forbidden variadic positional parameter")
+            raise TypeError("Forbidden variadic positional parameter")
         if param.kind == Parameter.VAR_KEYWORD:
-            fields[name] = field(default_factory=dict,  # type: ignore
-                                 metadata=properties())
+            fields[name] = field(default_factory=dict, metadata=properties(),)
             type_ = types.get(name, Any)
             annotations[name] = Mapping[str, type_]  # type: ignore
             kwargs = name
