@@ -1,33 +1,53 @@
-from dataclasses import Field as BaseField, is_dataclass
 from enum import Enum
-from typing import (Any, Callable, Dict, Iterable, List, Mapping, Optional,
-                    Sequence, Tuple, Type, TypeVar)
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Iterable,
+    List,
+    Mapping,
+    Optional,
+    Sequence,
+    Tuple,
+    Type,
+    TypeVar,
+)
+
+from dataclasses import Field as BaseField, is_dataclass
 
 from apischema.conversion import Converter, InputVisitorMixin
 from apischema.data.coercion import STR_NONE_VALUES, coerce
-from apischema.data.common_errors import bad_literal, wrong_type
 from apischema.dataclasses import Field, FieldKind, get_input_fields
 from apischema.ignore import Ignored
 from apischema.schema import Constraint, Schema
-from apischema.schema.constraints import (ArrayConstraint, ObjectConstraint,
-                                          get_constraint)
+from apischema.schema.constraints import (
+    ArrayConstraint,
+    ObjectConstraint,
+    get_constraint,
+)
 from apischema.types import ITERABLE_TYPES, MAPPING_TYPES, NoneType
-from apischema.utils import distinct
-from apischema.validation.errors import (ValidationError,
-                                         exception,
-                                         merge)
+from apischema.typing import _type_repr
+from apischema.validation.errors import ValidationError, exception, merge
 from apischema.validation.mock import ValidatorMock
 from apischema.validation.validator import Validator, get_validators, validate
 from apischema.visitor import Visitor
 
 
 def check_type(data: Any, expected: Type):
-    if type(data) is not expected:
-        raise ValidationError([wrong_type(type(data), expected)])
+    if not isinstance(data, expected):
+        raise ValidationError([f"expected type {_type_repr(expected)}"])
 
 
-def validate_with_errors(data: Any, constraint: Optional[Constraint],
-                         errors: Mapping[str, ValidationError]):
+def not_one_of(values: Iterable[Any]) -> ValidationError:
+    allowed_values = [
+        value if not isinstance(value, Enum) else value.value for value in values
+    ]
+    return ValidationError([f"not one of {allowed_values}"])
+
+
+def validate_with_errors(
+    data: Any, constraint: Optional[Constraint], errors: Mapping[str, ValidationError]
+):
     if constraint is not None:
         try:
             constraint.validate(data)
@@ -55,8 +75,9 @@ def apply_converter(value: From, converter: Callable[[From], To]) -> To:
 DataWithConstraint = Tuple[Any, Optional[Constraint]]
 
 
-class FromData(InputVisitorMixin[DataWithConstraint, Any],
-               Visitor[DataWithConstraint, Any]):
+class FromData(
+    InputVisitorMixin[DataWithConstraint, Any], Visitor[DataWithConstraint, Any]
+):
     def __init__(self, additional_properties: bool):
         Visitor.__init__(self)  # type: ignore
         self.additional_properties = additional_properties
@@ -68,8 +89,9 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
             constraint.validate(data)
         return data
 
-    def _union(self, alternatives: Iterable[Type], data2: DataWithConstraint
-               ) -> Tuple[Any, int]:
+    def _union(
+        self, alternatives: Iterable[Type], data2: DataWithConstraint
+    ) -> Tuple[Any, int]:
         error: Optional[ValidationError] = None
         for i, cls in enumerate(alternatives):
             try:
@@ -86,8 +108,9 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
             return None
         return self._union(alternatives, data2)[0]
 
-    def iterable(self, cls: Type[Iterable], value_type: Type,
-                 data2: DataWithConstraint):
+    def iterable(
+        self, cls: Type[Iterable], value_type: Type, data2: DataWithConstraint
+    ):
         data, constraint = data2
         check_type(data, list)
         elts: List[value_type] = []  # type: ignore
@@ -105,8 +128,13 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
         validate_with_errors(data, constraint, errors)
         return ITERABLE_TYPES[cls](elts)
 
-    def mapping(self, cls: Type[Mapping], key_type: Type,  # type: ignore
-                value_type: Type, data2: DataWithConstraint):
+    def mapping(
+        self,
+        cls: Type[Mapping],
+        key_type: Type,  # type: ignore
+        value_type: Type,
+        data2: DataWithConstraint,
+    ):
         data, constraint = data2
         check_type(data, dict)
         mapping: Dict[key_type, value_type] = {}  # type: ignore
@@ -119,15 +147,19 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
             assert isinstance(key, str)
             try:
                 new_key = self.visit(key_type, (key, None))
-                mapping[new_key] = self.visit(value_type,
-                                              (value, prop_constraint))
+                mapping[new_key] = self.visit(value_type, (value, prop_constraint))
             except ValidationError as err:
                 errors[key] = err
         validate_with_errors(data, constraint, errors)
         return MAPPING_TYPES[cls](mapping)
 
-    def typed_dict(self, cls: Type, keys: Mapping[str, Type],
-                   total: bool, data2: DataWithConstraint):
+    def typed_dict(
+        self,
+        cls: Type,
+        keys: Mapping[str, Type],
+        total: bool,
+        data2: DataWithConstraint,
+    ):
         data, constraint = data2
         check_type(data, dict)
         constraint = constraint or get_constraint(cls)
@@ -154,8 +186,7 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
     def tuple(self, types: Sequence[Type], data2: DataWithConstraint):
         data, constraint = data2
         check_type(data, list)
-        ArrayConstraint(min_items=len(types),
-                        max_items=len(types)).validate(data)
+        ArrayConstraint(min_items=len(types), max_items=len(types)).validate(data)
         elts: List[Any] = []
         errors: Dict[str, ValidationError] = {}
         elt_constraints = None
@@ -179,18 +210,25 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
 
     def literal(self, values: Sequence[Any], data2: DataWithConstraint):
         # Literal can contain Enum values and thus has to call visit too
-        assert data2[1] is None
-        for cls in distinct(map(type, values)):
-            try:
-                value = self.visit(cls, data2)
-            except ValidationError:
-                continue
-            if value in values:
-                return value
-        raise ValidationError([bad_literal(data2[0], values)])
+        data, _ = data2
+        assert _ is None
+        if data in values:
+            return data
+        enums = set()
+        for value in values:
+            if isinstance(value, Enum) and type(value) not in enums:
+                enums.add(type(value))
+                try:
+                    enum_data = self.visit(type(value), data2)
+                except ValidationError:
+                    continue
+                if enum_data in values:
+                    return enum_data
+        raise not_one_of(values)
 
-    def _custom(self, cls: Type, custom: Dict[Type, Converter],
-                data2: DataWithConstraint):
+    def _custom(
+        self, cls: Type, custom: Dict[Type, Converter], data2: DataWithConstraint
+    ):
         data, constraint = data2
         constraint = constraint or get_constraint(cls)
         types = list(custom)
@@ -239,8 +277,10 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
             additional = {key: data[key] for key in remain}
             set_field(additional_field, additional, "<additionalProperties>")
         elif remain and not self.additional_properties:
-            field_errors.update((field.alias, ValidationError(["field not allowed"]))
-                                for field in sorted(remain))
+            field_errors.update(
+                (field.alias, ValidationError(["field not allowed"]))
+                for field in sorted(remain)
+            )
         error: Optional[ValidationError] = None
         if field_errors:
             error = ValidationError(children=field_errors)
@@ -287,8 +327,8 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
         assert constraint is None
         try:
             return cls(data)
-        except ValueError as err:
-            raise ValidationError([str(err)])
+        except ValueError:
+            raise not_one_of(cls)
 
     def new_type(self, cls: Type, super_type: Type, data2: DataWithConstraint):
         data, constraint = data2
@@ -298,8 +338,9 @@ class FromData(InputVisitorMixin[DataWithConstraint, Any],
     def any(self, data2: DataWithConstraint):
         return data2[0]
 
-    def annotated(self, cls: Type, annotations: Sequence[Any],
-                  data2: DataWithConstraint):
+    def annotated(
+        self, cls: Type, annotations: Sequence[Any], data2: DataWithConstraint
+    ):
         if Ignored in annotations:
             raise Ignored()
         data, constraint = data2
@@ -330,12 +371,19 @@ class FromDataWithCoercion(FromData):
         return data
 
     def enum(self, cls: Type[Enum], data2: DataWithConstraint):
-        data, constraint = data2
-        data = self.primitive(type(data), (data, None))
-        return super().enum(cls, (data, constraint))
+        data, _ = data2
+        assert _ is None
+        if not isinstance(cls, data):
+            data = self.visit(type(data), data2)
+        return super().enum(cls, (data, None))
 
 
-def from_data(cls: Type[T], data: Any, *, additional_properties: bool = False,
-              coerce: bool = False) -> T:
+def from_data(
+    cls: Type[T],
+    data: Any,
+    *,
+    additional_properties: bool = False,
+    coerce: bool = False,
+) -> T:
     visitor = FromData if not coerce else FromDataWithCoercion
     return visitor(additional_properties).visit(cls, (data, None))
