@@ -1,6 +1,16 @@
+from typing import (
+    Any,
+    Callable,
+    Dict,
+    Generator,
+    Generic,
+    Hashable,
+    Iterable,
+    TypeVar,
+    Union,
+)
+
 from dataclasses import fields, is_dataclass
-from typing import (Any, Callable, Dict, Generator, Generic, Hashable, Iterable,
-                    TypeVar, Union)
 
 PREFIX = "_apischema_"
 NO_DEFAULT = object()
@@ -10,21 +20,11 @@ Func = TypeVar("Func", bound=Callable)
 Hashable_ = TypeVar("Hashable_", bound=Hashable)
 
 
-def distinct(values: Iterable[Hashable_]) -> Iterable[Hashable_]:
-    unique = set()
-    for value in values:
-        if value not in unique:
-            unique.add(value)
-            yield value
-
-
-def to_hashable(data: Union[None, int, float, str, bool, list, dict]
-                ) -> Hashable:
+def to_hashable(data: Union[None, int, float, str, bool, list, dict]) -> Hashable:
     if isinstance(data, list):
         return tuple(map(to_hashable, data))
     if isinstance(data, dict):
-        return tuple(sorted((to_hashable(k), to_hashable(v))
-                            for k, v in data.items()))
+        return tuple(sorted((to_hashable(k), to_hashable(v)) for k, v in data.items()))
     return data  # type: ignore
 
 
