@@ -31,16 +31,12 @@ def test_mock():
     assert mock.a == 0
     assert mock.b == "1"
     assert mock.c == 42
+    assert mock.d == 0
     assert mock.__class__ == Data
-    assert mock.__dict__ == {"a":             0, "b": "1",
-                             FIELDS_SET_ATTR: {"a"}}
+    assert mock.__dict__ == {"a": 0, "b": "1", FIELDS_SET_ATTR: {"a"}}
     assert mock.property == 1
     assert mock.method(1) == 1
     assert mock.classmethod(0) == 42
     assert type(mock) == ValidatorMock
-    # limitation of mocking, class variables must be typed
-    # in order to be detected
-    with raises(NonTrivialDependency):
-        _ = mock.d
     with raises(NonTrivialDependency):
         _ = mock.e
